from smtplib import SMTP
import logging


EMAIL_TEMPLATE = """\
From: {email}
To: {email}
Subject: {subject}

{message}
"""


def notify(email, pwd, subject, message, to=None):
    # TODO: add server as parameter
    if to is None:
        to = [email]
    with SMTP("smtp.gmail.com", 587) as server:
        server.starttls()
        server.login(email, pwd)
        email_text = EMAIL_TEMPLATE.format(
            email=email, subject=subject, message=message
        ).encode("utf8")
        server.sendmail(email, to, email_text)
        logging.info("sent email to %s with message: %s", email, message)



import functools
from pprint import pprint
import sys
from time import sleep

import click

from .bot import Bot
from .notifs import notify
from .config import get_config
from . import logging


def catch_exceptions(config):
    def catch_exceptions_decorator(fun):
        @functools.wraps(fun)
        def wrapper(*args, **kwargs):
            try:
                return fun(*args, **kwargs)
            except Exception:
                logging.exception(fun.__name__)
                import traceback

                if config["notify_on_error"]:
                    notify(
                        email=config["email"]["login"],
                        pwd=config["email"]["password"],
                        subject="ERREUR STATIONNEMENT",
                        message=traceback.format_exc(),
                        to=config["notify_on_error"],
                    )

        return wrapper

    return catch_exceptions_decorator


def connect(config):
    pbp_login = config["paybyphone"]["login"]
    pbp_pwd = config["paybyphone"]["password"]
    bot = Bot(pbp_login, pbp_pwd)
    logging.info("created bot")
    return bot


@click.group()
def cli():
    pass


@cli.command()
@click.argument("config_name")
@click.option("--config", required=False, type=str)
def payment_accounts(config_name, config):
    """
    Show the payment accounts
    """
    config = get_config(config_name, config)
    bot = connect(config)
    pprint(bot.get_payment_accounts())


@cli.command()
@click.argument("config_name")
@click.option("--location", default=None)
@click.option("--config", required=False, type=str)
def check(config_name, location, config):
    """
    Check if there is an ongoing subscription
    """
    config = get_config(config_name, config)

    def _check(config, location):
        bot = connect(config)
        sessions = bot.get_parking_sessions(
            licensePlate=config["plate"], locationId=location
        )
        logging.info(
            "retrieved parking sessions with filter licensePlate=%s, locationId=%s: %s"
            % (config["plate"], location, sessions)
        )
        if sessions:
            print("Found sessions", file=sys.stderr)
            pprint(sessions)
        else:
            print("No session", file=sys.stderr)

    catch_exceptions(config)(_check)(config, location)


@cli.command()
@click.argument("config_name")
@click.option("--location", default=None)
@click.option("--config", required=False, type=str)
def alert(config_name, location, config):
    """
    1. Check if there is an ongoing subscription

    2. Else send a notification
    """

    config = get_config(config_name, config)

    def _alert(config, location):
        bot = connect(config)
        sessions = bot.get_parking_sessions(
            licensePlate=config["plate"], locationId=location
        )
        logging.info(
            "retrieved parking sessions with filter licensePlate=%s, locationId=%s: %s"
            % (config["plate"], location, sessions)
        )
        if sessions:
            print("Found sessions", file=sys.stderr)
            pprint(sessions)
        else:
            print("No session, sending email to", config["notify"], file=sys.stderr)
            notify(
                email=config["email"]["login"],
                pwd=config["email"]["password"],
                subject="ALERTE STATIONNEMENT",
                message=(
                    "Aucun stationnement en cours !!!\n"
                    "Pour le renouveller : https://m2.paybyphone.fr/parking"
                ),
                to=config["notify"],
            )

    catch_exceptions(config)(_alert)(config, location)


@cli.command()
@click.argument("config_name")
@click.option("--location", required=True, type=str)
@click.option("--rate", required=True, type=int)
@click.option("--duration", required=True, type=str)
@click.option("--config", required=False, type=str)
def pay(config_name, location, rate, duration, config):
    """
    1. Check if there is an ongoing subscription

    2. Else pay

    3. Check that the payment succeeded

    4. Notify on failure
    """
    config = get_config(config_name, config)

    def _pay(config, location, duration):
        bot = connect(config)
        logging.info("launching payment")
        sessions = bot.get_parking_sessions(
            licensePlate=config["plate"], locationId=location
        )
        if sessions:
            print("Already registered", file=sys.stderr)
            pprint(sessions)
            return
        bot.pay(
            durationQuantity=duration,
            durationTimeUnit="Days",
            licensePlate=config["plate"],
            locationId=location,
            rateOptionId=rate,
            paymentAccountId=config["paymentAccountId"],
        )
        sleep(20)  # wait for the payment to be executed
        sessions = bot.get_parking_sessions(
            licensePlate=config["plate"], locationId=location
        )
        if sessions:
            print("Payment done", file=sys.stderr)
            pprint(sessions)
        else:
            print("Payment failed", file=sys.stderr)
            notify(
                email=config["email"]["login"],
                pwd=config["email"]["password"],
                subject="ALERTE STATIONNEMENT",
                message=(
                    "Aucun stationnement en cours !!!\n"
                    "Pour le renouveller : https://m2.paybyphone.fr/parking"
                ),
                to=config["notify"],
            )

    catch_exceptions(config)(_pay)(config, location, duration)


if __name__ == "__main__":
    cli()



from datetime import datetime
import re

import requests


class Bot:
    base_headers = {
        "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36",
        "Accept-Language": "en-US,en;q=0.5",
        "Accept-Encoding": "gzip, deflate, br",
        "Referer": "https://m2.paybyphone.fr/",
        "Origin": "https://m2.paybyphone.fr",
        "DNT": "1",
        "Connection": "keep-alive",
    }

    def __init__(self, username, password):
        url = "https://m2.paybyphone.fr/static/js/main.0aec44c0.chunk.js"
        try:
            resp = requests.get(url)
            resp.raise_for_status()
            pattern = r'paymentService:{[^}]*apiKey:\"(.*?)\"'
            result = re.search(pattern, resp.text, flags=re.MULTILINE)
            self.apiKey = result.group(1) if result else None
        except (requests.exceptions.HTTPError, KeyError):
            self.apiKey = None

        r = requests.post(
            "https://auth.paybyphoneapis.com/token",
            headers={
                **self.base_headers,
                "Accept": "application/json, text/plain, */*",
                "X-Pbp-ClientType": "WebApp",
            },
            data={
                "grant_type": "password",
                "username": username,
                "password": password,
                "client_id": "paybyphone_webapp",
            },
        )
        j = r.json()
        self.authorization = j["token_type"] + " " + j["access_token"]

        r = self._get("https://consumer.paybyphoneapis.com/parking/accounts")
        j = r.json()
        self.parkingAccountId = j[0]["id"]

    def _get_parking_sessions(self):
        ans = self._get(
            "https://consumer.paybyphoneapis.com/parking/accounts/%s/sessions"
            % self.parkingAccountId,
            params={"periodType": "Current"},
        ).json()
        for s in ans:
            s["startTime"] = datetime.strptime(s["startTime"], "%Y-%m-%dT%H:%M:%SZ")
            s["expireTime"] = datetime.strptime(s["expireTime"], "%Y-%m-%dT%H:%M:%SZ")
        return ans

    def get_parking_sessions(self, licensePlate=None, locationId=None):
        def pred(p):
            if licensePlate is not None:
                if p["vehicle"]["licensePlate"] != licensePlate:
                    return False
            if locationId is not None:
                if p["locationId"] != locationId:
                    return False
            return True

        return list(filter(pred, self._get_parking_sessions()))

    def last_expiry(self, licensePlate, locationId):
        return max(
            (
                session["expireTime"]
                for session in self.get_parking_sessions(licensePlate, locationId)
            ),
            default=None,
        )

    def get_payment_accounts(self):
        return self._get("https://payments.paybyphoneapis.com/v1/accounts").json()

    def _get_rate_options(self, location, licensePlate):
        return self._get(
            "https://consumer.paybyphoneapis.com/parking/locations/%s/rateOptions" % location,
            params={
                "parkingAccountId": self.parkingAccountId,
                "licensePlate": licensePlate,
            },
        ).json()

    def _get_rate_options_renew(self, location, parkingSessionId):
        return self._get(
            "https://consumer.paybyphoneapis.com/parking/locations/%s/rateOptions" % location,
            params={
                "parkingAccountId": self.parkingAccountId,
                "parkingSessionId": parkingSessionId,
            },
        ).json()

    def _get_quote(
        self, durationQuantity, durationTimeUnit, licensePlate, locationId, rateOptionId
    ):
        return self._get(
            "https://consumer.paybyphoneapis.com/parking/accounts/%s/quote"
            % self.parkingAccountId,
            params={
                "durationQuantity": durationQuantity,
                "durationTimeUnit": durationTimeUnit,
                "licensePlate": licensePlate,
                "locationId": locationId,
                "rateOptionId": rateOptionId,
            },
        ).json()

    def _get_renew_quote(self, durationQuantity, durationTimeUnit, parkingSessionId):
        return self._get(
            "https://consumer.paybyphoneapis.com/parking/accounts/%s/quote"
            % self.parkingAccountId,
            params={
                "durationQuantity": durationQuantity,
                "durationTimeUnit": durationTimeUnit,
                "parkingSessionId": parkingSessionId,
            },
        )

    def _post_quote(
        self,
        quoteId,
        paymentAccountId,
        licensePlate,
        rateOptionId,
        locationId,
        durationQuantity,
        durationTimeUnit,
        startTime,
    ):
        return self._post(
            "https://consumer.paybyphoneapis.com/parking/accounts/%s/sessions/"
            % self.parkingAccountId,
            json={
                "licensePlate": licensePlate,
                "locationId": locationId,
                "stall": None,
                "rateOptionId": rateOptionId,
                "startTime": startTime,  # found in parkingStartTime of quote
                "quoteId": quoteId,
                "duration": {
                    "timeUnit": durationTimeUnit,
                    "quantity": durationQuantity,
                },
                "paymentMethod": {
                    "paymentMethodType": "PaymentAccount",
                    "payload": {"paymentAccountId": paymentAccountId, "cvv": None},
                },
            },
        )

    def _put_renew_quote(
        self,
        parkingSessionId,
        quoteId,
        paymentAccountId,
        durationQuantity,
        durationTimeUnit,
    ):
        r = self._put(
            "https://consumer.paybyphoneapis.com/parking/accounts/%s/sessions/%s"
            % (self.parkingAccountId, parkingSessionId),
            data={
                "duration": {
                    "timeUnit": durationTimeUnit,
                    "quantity": durationQuantity,
                },
                "quoteId": quoteId,
                "paymentMethod": {
                    "paymentMethodType": "PaymentAccount",
                    "payload": {"paymentAccountId": paymentAccountId, "cvv": None},
                },
            },
        )
        assert r.status_code == requests.codes["accepted"]

    def _get_workflow(self, quoteId):
        return self._get("https://consumer.paybyphoneapis.com/events/workflow/%s" % quoteId)

    def pay(
        self,
        durationQuantity,
        durationTimeUnit,
        licensePlate,
        locationId,
        rateOptionId,
        paymentAccountId,
    ):
        quote = self._get_quote(
            durationQuantity=durationQuantity,
            durationTimeUnit=durationTimeUnit,
            licensePlate=licensePlate,
            locationId=locationId,
            rateOptionId=rateOptionId,
        )
        quoteId = quote["quoteId"]
        startTime = quote["parkingStartTime"]

        post = self._post_quote(
            quoteId=quoteId,
            paymentAccountId=paymentAccountId,
            licensePlate=licensePlate,
            rateOptionId=rateOptionId,
            locationId=locationId,
            durationQuantity=durationQuantity,
            durationTimeUnit=durationTimeUnit,
            startTime=startTime,
        )
        return post.text

    def _request(self, method, url, **kwargs):
        return requests.request(
            method,
            url,
            headers={
                **self.base_headers,
                "Accept": "application/json, text/plain, */*",
                "x-pbp-version": "2",
                "x-api-key": self.apiKey,
                "Authorization": self.authorization,
            },
            **kwargs
        )

    def _get(self, url, **kwargs):
        return self._request("GET", url, **kwargs)

    def _post(self, url, **kwargs):
        return self._request("POST", url, **kwargs)

    def _put(self, url, **kwargs):
        return self._request("PUT", url, **kwargs)



from pathlib import Path
import logging

import yaml


def get_config(config_name, config=None):
    if config:
        with open(config) as ymlfile:
            config = yaml.safe_load(ymlfile)
    else:
        yml_path = Path("~/.config/paybybot3.yml").expanduser()
        if yml_path.exists():
            with yml_path.open() as ymlfile:
                config = yaml.safe_load(ymlfile)
        else:
            logging.warning("~/.config/paybybot3.yml doesn't exist")
            raise FileNotFoundError("Either pass a config file or create ~/.config/paybybot3.yml")
    return config.get(config_name, {})



#!/usr/bin/env python3

import os
from setuptools import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="paybybot3",
    version="0.0.32",
    author="Louis Abraham (louisabraham) and updated by rklf",
    license="MIT",
    author_email="louis.abraham@yahoo.fr",
    description="CLI interface to https://www.paybyphone.fr/",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    url="https://github.com/rklf/paybybot3",
    packages=["paybybot3"],
    install_requires=["pyyaml", "requests", "click"],
    python_requires=">=3.6",
    entry_points={"console_scripts": ["paybybot3 = paybybot3.__main__:cli"]},
    classifiers=["Topic :: Utilities"],
)
